/*  GWhere.
 *  Copyright (C) 2000  Sébastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/*!
 * @header	gwgui.h
 *		GWhere<br>
 *		Copyright (C) 2000  Sébastien LECACHEUR<br><br>
 *		This Distributed under the terms of the <a href="http://www.gnu.org/copyleft/gpl.html">GNU General Public Licence</a><br><br>
 *		This component was developed for the GWhere project.<br><br>
 * @abstract	This is the main header for GUI components library.
 * @author	Sébastien LECACHEUR <zero@gwhere.org>
 * @version	1.0 2003/02/06
 * @see		gwsupport.h
 * @link	GWhere Web Site <http://www.gwhere.org>
 * @author	Author <zero@gwhere.org>
 */


#ifndef GW_GUI_H
#define GW_GUI_H


#include <gtk/gtk.h>

#include "gwcapturebox.h"
#include "gwcategorieseditbox.h"
#include "gwcomboboxcategories.h"
#include "gwdialogbox.h"
#include "gwfileselectionbox.h"
#include "gwmenu.h"
#include "gwmenuaction.h"
#include "gwmenuactioncallback.h"
#include "gwmenufile.h"
#include "gwmenufilecallback.h"
#include "gwmenuhelp.h"
#include "gwmenuhelpcallback.h"
#include "gwmenuoptions.h"
#include "gwmenuoptionscallback.h"
#include "gwmenupopup.h"
#include "gwmenupopupcallback.h"
#include "gwmsgbox.h"
#include "gwnotebook.h"
#include "gwnotebookcallback.h"
#include "gwnotebookcatalog.h"
#include "gwnotebookcatalogcallback.h"
#include "gwnotebookmanagment.h"
#include "gwnotebookmanagmentcallback.h"
#include "gwnotebooksearch.h"
#include "gwnotebooksearchcallback.h"
#include "gwprogressbarbox.h"
#include "gwpropertiesbox.h"
#include "gwpropertiesboxcallback.h"
#include "gwsettingscatalogfile.h"
#include "gwsettingsexplorer.h"
#include "gwsettingsexplorerdisk.h"
#include "gwsettingsexplorerfile.h"
#include "gwsettingsgeneral.h"
#include "gwsettingsgui.h"
#include "gwsettingsinterface.h"
#include "gwsettingsmail.h"
#include "gwsettingswindowbox.h"
#include "gwsettingswindowboxcallback.h"
#include "gwstatusbar.h"
#include "gwstatusbarcallback.h"
#include "gwtextbox.h"
#include "gwtoolbar.h"
#include "gwwindowboxabout.h"
#include "gwwindowboxmail.h"
#include "gwwindowboxmailcallback.h"


#endif
