GPL

Introduction

This is an unofficial translation of the GNU General Public License
into Polish. It was not published by the Free Software Foundation, and
does not legally state the distribution terms for software that uses
the GNU GPL--only the original English text of the GNU GPL does that.
However, we hope that this translation will help Polish speakers
understand the GNU GPL better.

Source is http://www.linux.org.pl/gpl.php

    Powszechna Licencja Publiczna GNU
       Wersja 2, czerwiec 1991

 Copyright © 1989, 1991 Free Software Foundation, Inc., 675 Mass Ave,
 Cambridge, MA 02139 USA.
 Zezwala sie na kopiowanie i rozpowszechnianie wiernych kopii niniejszego
 dokumentu licencyjnego, jednak bez prawa wprowadzania zmian.

    Preambula

  Wiekszosc licencji na oprogramowanie pomyslana jest po to, aby
odebrac uzytkownikowi mozliwosc swobodnego udostepniania innym i
zmieniania danego software'u. Natomiast w wypadku Powszechnej Licencji
Publicznej GNU (GNU General Public License, GPL) celem jest
zagwarantowanie uzytkownikowi swobody udostepniania i zmieniania tego
bezplatnego oprogramowania, a wiec danie pewnosci, iz oprogramowanie
jest wolno dostepne dla wszystkich uzytkowników. Niniejsza Powszechna
Licencja Publiczna dotyczy wiekszosci oprogramowania wydawanego przez
Fundacje Wolnego Oprogramowania (Free Software Foundation) oraz
wszelkich innych programów, których autorzy zobowiazuja sie do jej
stosowania. (Niektóre rodzaje oprogramowania wydawanego przez Fundacje
objete sa Powszechna Licencja Publiczna GNU dla Bibliotek, GNU Library
General Public License). Uzytkownik moze stosowac ja równiez do swoich
programów. 

  Mówiac o wolnym oprogramowaniu mamy na mysli swobode, a nie cene.
Nasze Powszechne Licencje Publiczne wprowadzono po to, aby zapewnic
Panstwu swobode rozpowszechniania kopii tego oprogramowania (i - jesli
ktos chce - pobierania za te usluge oplaty), jak równiez aby udostepnic
kod zródlowy oraz umozliwic dokonywanie zmian tego oprogramowania lub
wykorzystywania jego fragmentów w nowych, wolnych programach. Nie bez
znaczenia jest tez sama mozliwosc dotarcia do Panstwa z informacja o
wszystkich tych udogodnieniach.

  W celu ochrony praw uzytkownika jestesmy zmuszeni wprowadzac
ograniczenia zabraniajace komukolwiek kwestionowanie jego praw albo
sugerowanie rezygnacji z tych praw. Ograniczenia te sprowadzaja sie
do pewnych dla Panstwa obowiazków w przypadku rozpowszechniania przez
Was kopii naszego oprogramowania badz dokonywania w nim zmian. 

  Na przyklad, jesli rozprowadzacie Panstwo kopie takiego programu,
niezaleznie czy gratisowo, czy za oplata, musicie Panstwo odbiorcy
udzielic wszelkich tych praw, jakie mieliscie sami. Musicie zapewnic
mu równiez otrzymanie kodu zródlowego (lub mozliwosc otrzymania) oraz
przedstawic niniejsze Warunki, aby mógl on poznac swoje prawa. 

  Ochrona Panstwa praw przebiega w dwóch etapach: (1) zastrzegamy
prawo wlasnosci autorskiej do oprogramowania, (2) oferujemy Panstwu
niniejsza licencje, która daje Wam sankcjonowane prawem zezwolenie na
kopiowanie, rozpowszechnianie i/lub modyfikowanie tego oprogramowania.

  Ponadto dla ochrony tak autora, jak i naszej, pragniemy miec pewnosc,
ze kazdy zrozumie, iz na niniejsze wolne oprogramowanie nie udziela
sie gwarancji. W razie dokonania w nim przez kogos modyfikacji i
puszczenia dalej do obrotu, pragniemy, aby dalsi odbiorcy zdawali
sobie sprawe z tego, ze problemy wprowadzone przez inne osoby nie sa
wyrazem oryginalnych dzialan twórców.

  I rzecz ostatnia: kazdemu wolnemu programowi stale zagrazaja patenty
na oprogramowanie. Naszym pragnieniem jest unikanie takiego
niebezpieczenstwa, kiedy redystrybutorzy wolnego programu
indywidualnie uzyskuja prawa patentowe, nadajac tym samym programowi
charakter prawnie zastrzezony. W celu zapobiezenia takim zjawiskom
jednoznacznie wyjasnilismy, ze kazdy patent musi byc wydawany albo dla
swobodnego uzytku przez wszystkich, albo nie wydawany wcale. 

  Ponizej podajemy dokladne zasady i warunki kopiowania,
rozpowszechniania i modyfikowania. 

  Zasady i warunki kopiowania, rozpowszechniania i modyfikowania

  0. Niniejsza Licencja dotyczy programów i innych prac, na których
umieszczona jest pochodzaca od wlasciciela praw autorskich informacja,
ze dany program lub praca moze byc rozpowszechniana na warunkach
niniejszej Powszechnej Licencji Publicznej. Uzywane ponizej slowo
"Program" oznacza wlasnie takie programy lub prace, zas okreslenie
"praca oparta na Programie" dotyczy albo Programu, albo pochodzacej od
niego pracy w rozumieniu prawa autorskiego, to jest pracy zawierajacej
Program lub jego czesc doslowna, badz zmodyfikowana i/lub przelozona
na inny jezyk. (W dalszym ciagu niniejszego, pojecie przekladu wlacza
sie bez ograniczen do terminu "modyfikacja"). Do kazdego licencjobiorcy
bedziemy zwracac sie "per Ty".

Niniejsza Licencja nie obejmuje dzialan innych niz kopiowanie,
rozprowadzanie i modyfikowanie - nie mieszcza sie one w jej zakresie.
Czynnosc uzywania Programu nie jest poddana ograniczeniom, a produkty
uzyskane z Programu objete sa Licencja tylko wtedy, gdy ich tresc
stanowi prace oparta na Programie (niezaleznie od stworzenia jej przy
uzyciu Programu). To, czy fakt taki jest prawda, zalezy od tego, co
dany Program wykonuje.

  1. Mozesz kopiowac i rozprowadzac w dowolnych mediach wierne kopie
kodu zródlowego Programu w otrzymanej formie pod warunkiem, ze w widoczny
sposób i odpowiednio podasz na kazdej kopii wlasciwa informacje o prawie
autorskim i zrzeczenie sie uprawnien z tytulu gwarancji; wszelkie napisy
informacyjne na temat Licencji i faktu nieudzielania gwarancji musisz
chronic przed uszkodzeniem, zas wszystkim innym odbiorcom Programu
musisz wraz z Programem wreczac egzemplarz niniejszej Licencji.

Mozesz pobierac oplate za fizyczna czynnosc przekazania kopii i wedlug
wlasnej decyzji mozesz za oplata proponowac ochrone gwarancyjna. 

  2. Mozesz modyfikowac swoja kopie czy kopie Programu oraz dowolne
jego czesci, tworzac przez to prace oparta na Programie, jak równiez
kopiowac i rozprowadzac takie modyfikacje i prace na warunkach
podanych w pkt.1 powyzej - pod warunkiem przestrzegania calosci
ponizszych wymogów:

    a) Musisz spowodowac umieszczenie na zmodyfikowanych plikach 
    widocznej informacji o tym, ze dane pliki zostaly przez ciebie
    zmienione, wraz z data dokonania zmian.

    b) Musisz doprowadzic do tego, aby kazda rozpowszechniana lub
    publikowana przez ciebie praca, która w calosci lub czesci zawiera
    Program, albo pochodzi od niego lub jego czesci, byla w calosci i
    bezplatnie licencjonowana dla wszelkich stron trzecich na
    warunkach niniejszej Licencji.

    c) Jezeli zmodyfikowany program podczas korzystania z niego w
    normalnym trybie odczytuje polecenia interaktywnie, musisz
    spowodowac, aby po uruchomieniu (uzyty w interaktywny sposób w
    najzwyklejszym trybie), wydrukowywal on lub wyswietlal
    powiadomienie o odnosnym prawie autorskim i braku gwarancji
    (ewentualnie o zapewnianiu gwarancji przez ciebie), oraz o tym, ze
    uzytkownicy moga redystrybuowac ten program na niniejszych
    warunkach wraz z informacja, jak uzytkownik moze zapoznac sie z
    trescia niniejszej Licencji. (Wyjatek: jesli sam Program jest
    interaktywny, ale normalnie nie drukuje takiego powiadomienia,
    twoja praca oparta na nim tez nie musi wydrukowywac takiego
    powiadomienia).

Niniejsze wymogi odnosza sie do zmodyfikowanej pracy jako calosci.
Jesli dajace sie ustalic sekcje danej pracy nie pochodza od Programu i
moga byc racjonalnie uwazane za samodzielne i odrebne same w sobie, to
niniejsza Licencja i jej warunki nie maja zastosowania do takich sekcji
przy rozprowadzaniu ich przez ciebie jako odrebne prace. Jesli jednak
rozprowadzasz je jako czesc calosci, bedacej praca oparta na Programie,
rozpowszechnianie tej calosci musi byc dokonywane na warunkach
niniejszej Licencji, której zezwolenia dla innych licencjobiorców
rozciagaja sie w calej szerokosci na te calosc, a tym samym i na kazda
indywidualna jej czesc, niezaleznie od jej autorstwa. 

Dlatego tez intencja tego fragmentu nie jest roszczenie sobie praw albo
podwazanie twych praw do pracy napisanej w calosci przez ciebie. Chodzi
nam raczej o korzystanie z prawa kontrolowania dystrybucji pochodnych i
zbiorowych prac opartych na Programie. 

I jeszcze jedno: samo tylko polaczenie z Programem (lub z praca oparta
na Programie) innej pracy - nie opartej na Programie, w ramach wolumenu
nosnika przechowywania lub dystrybucji, nie powoduje objecia takiej
pracy zakresem niniejszej Licencji. 

  3. Mozesz kopiowac i rozprowadzac Program (lub oparta na nim prace -
zgodnie z pkt.2 w kodzie wynikowym lub w formie wykonywalnej w mysl
postanowien pkt.1 i 2 powyzej, pod warunkiem zrealizowania równiez
ponizszych wymogów:

    a) Musisz dolaczyc do niego odpowiadajacy mu, kompletny i mozliwy
    do odczytania przez urzadzenia cyfrowe kod zródlowy, który musi byc
    rozpowszechniany na warunkach pkt.1 i 2 powyzej i na nosniku
    zwyczajowo uzywanym dla wzajemnej wymiany oprogramowania; lub

    b) dolaczyc do niego pisemna oferte, wazna co najmniej 3 lata,
    przyznajaca kazdej stronie trzeciej - za oplata nie przekraczajaca
    twego kosztu fizycznego wykonywania dystrybucji zródla - kompletna,
    odczytywalna przez urzadzenia cyfrowe kopie odpowiadajacego mu kodu
    zródlowego, rozprowadzana na warunkach pkt.1 i 2 powyzej, na nosniku
    zwyczajowo uzywanym do wzajemnej wymiany oprogramowania; lub

    c) dolaczyc do niego informacje, jaka otrzymales na temat oferty
    rozprowadzania odpowiedniego kodu zródlowego. (Ta mozliwosc
    dozwolona jest tylko dla dystrybucji niehandlowej i jedynie wtedy,
    gdy otrzymales dany program w kodzie wynikowym lub formie
    wykonywalnej wraz z wymieniona oferta - zgodnie z podpunktem "b"
    powyzej).

Okreslenie kod zródlowy dla pracy oznacza forme pracy preferowana dla
wprowadzania do niej modyfikacji. Dla wykonanej pracy, kompletny kod
zródlowy oznacza caly kod zródlowy wszystkich modulów, wszelkie
sprzezone z nia posredniczace pliki opisujace oraz zbiory komend
stosowane do sterowania kompilacja i instalowaniem programów. Niemniej
jednak, jako wyjatek specjalny, dystrybuowany (w formie zródlowej albo
binarnej) kod zródlowy nie musi obejmowac niczego, co jest normalnie
rozprowadzane przy pomocy glównych komponentów (kompilator, jadro
itd.) systemu operacyjnego, na którym pracuje czesc wykonywalna, o ile
sam taki komponent towarzyszy tej czesci. 

Jesli dystrybucja czesci wykonywalnej albo kodu wynikowego realizowana
jest poprzez oferowanie dostepu do kopii z wyznaczonego miejsca, to
oferowanie równowaznego dostepu dla kopiowania kodu zródlowego z tego
samego miejsca liczy sie jako rozpowszechnianie kodu zródlowego, nawet
gdy strony trzecie nie sa zmuszone do kopiowania zródla wraz z kodem
wynikowym. 

  4. Poza przypadkami jednoznacznie dozwolonymi w niniejszej Licencji,
nie mozesz kopiowac, modyfikowac, sublicencjonowac ani
rozpowszechniac Programu. We wszystkich pozostalych wypadkach, kazda
próba skopiowania, sublicencjonowania lub rozpowszechnienia Programu
jest niewazna i powoduje automatyczne wygasniecie twoich praw z
tytulu Licencji. Niemniej jednak, stronom, które juz otrzymaly od
ciebie kopie albo prawa w ramach niniejszej Licencji, licencje nie
wygasaja tak dlugo, jak dlugo strony te w pelni stosuja sie do nich.

  5. Nie musisz akceptowac niniejszej Licencji, jezeli jej nie
podpisales. Niemniej jednak, nic innego nie zapewni ci zezwolenia na
modyfikowanie lub rozprowadzanie Programu i pochodzacych od niego
prac. Dzialania takie sa prawnie zabronione, jezeli nie przyjmujesz
niniejszej Licencji. Dlatego tez, poprzez modyfikowanie badz
rozpowszechnianie Programu (lub pracy na nim opartej) dajesz wyraz
swojej akceptacji dla Licencji i wszelkich jej postanowien i warunków
dotyczacych kopiowania, rozprowadzania i modyfikowania Programu lub
opartych na nim prac.

  6. W kazdym przypadku redystrybucji przez ciebie Programu (albo
opartej na nim pracy), odbiorca automatycznie otrzymuje od pierwotnego
licencjodawcy licencje na kopiowanie, rozpowszechnianie i modyfikowanie
Programu na niniejszych zasadach i warunkach. Na korzystanie przez
odbiorce z udzielonych w niniejszej Licencji praw nie mozesz narzucac
juz dalszych ograniczen. Nie jestes strona odpowiedzialna za kontrole
przestrzegania Licencji przez osoby trzecie.

  7. Jesli na skutek wyroku sadowego lub zarzutu naruszenia patentu,
jak tez z kazdej innej przyczyny (nie ograniczonej do kwestii
patentowych) zostana narzucone na ciebie (niezaleznie czy to moca
wyroku sadowego, umowy, czy w inny sposób) warunki sprzeczne z
warunkami niniejszej Licencji, to nie zwalniaja one ciebie z warunków
Licencji. Jesli nie mozesz prowadzic dystrybucji tak, aby wypelniac
jednoczesnie swoje obowiazki z tytulu niniejszej Licencji i inne
odnosne obowiazki, to w rezultacie nie mozesz wcale rozprowadzac
Programu. Na przyklad, gdyby licencja patentowa nie zezwalala na wolna
od oplat licencyjnych redystrybucje Programu przez wszystkie osoby,
które otrzymaly kopie bezposrednio lub posrednio od ciebie, to jedynym
sposobem pozwalajacym ci na przestrzeganie i licencji patentowej, i
Licencji niniejszej, byloby calkowite powstrzymanie sie od
jakiejkolwiek dystrybucji Programu.

Jezeli w jakichs szczególnych okolicznosciach którys fragment
niniejszego punktu stalby sie niewazny lub niewykonywalny, to intencja
jest, aby znajdowala zastosowanie pozostala czesc punktu, a tresc
calego punktu byla stosowana w pozostalych okolicznosciach.

Celem niniejszego punktu nie jest zachecanie do naruszania patentów
czy innych praw wlasnosci, albo tez do podwazania ich waznosci;
niniejszy punkt za swój jedyny cel ma ochrone integralnosci systemu
rozpowszechniania wolnego oprogramowania, realizowanego za pomoca
publicznych licencji. Wielu ludzi bezinteresownie wnioslo swój wklad
do stworzenia szerokiego zakresu oprogramowania upowszechnianego w tym
systemie, majac zaufanie do konsekwentnego jego stosowania; wylacznie
do autora/ofiarodawcy nalezy decyzja, czy zyczy on sobie rozprowadzania
oprogramowania za posrednictwem innego systemu i licencjobiorca nie
moze tego prawa wyboru ograniczac. 

Intencja niniejszego punktu jest jasne i wyrazne przedstawienie tego,
co uwaza sie za skutki, jakie rodzi pozostala czesc niniejszej Licencji. 

  8. W przypadku, gdy dystrybucja i/lub uzywanie Programu w niektórych
krajach poddane jest ograniczeniom patentowym lub zastrzezeniom prawami
autorskimi, poczatkowy posiadacz praw autorskich, który poddaje Program
pod oddzialywanie niniejszej Licencji, moze dodac wyraznie zakreslone
geograficzne ograniczenie rozpowszechniania wylaczajace te kraje,
dzieki czemu dystrybucja dozwolona bedzie wylacznie w krajach czy wsród
krajów nie objetych takim wylaczeniem. W przypadku takim, niniejsza
Licencja obejmuje dane ograniczenie tak, jakby bylo ono wpisane w jej
tresc.

  9. W miare potrzeby Fundacja Wolnego Oprogramowania moze publikowac
poprawione i/lub nowe wersje Powszechnej Licencji Publicznej. Takie
nowe wersje beda napisane w duchu podobnym do obecnej wersji, ale moga
róznic sie w szczególach poruszajacych nowe problemy czy zagadnienia.

Kazdej wersji nadaje sie wyrózniajacy ja numer. Jezeli Program podaje
numer wersji niniejszej Licencji, odnoszacy sie do tej wersji i
"wszelkich wersji nastepnych", masz do wyboru albo stosowac sie do
postanowien i warunków tej wersji, albo którejkolwiek wersji
pózniejszej wydanej przez Fundacje Wolnego Oprogramowania. O ile
Program nie podaje numeru wersji niniejszej Licencji, mozesz wybrac
dowolna wersje kiedykolwiek opublikowana przez Fundacje.

  10. Jesli chcesz wlaczyc czesci Programu do innych wolnych programów,
których warunki rozpowszechniania sa inne, zwróc sie pisemnie do autora
z prosba o pozwolenie. W przypadku oprogramowania objetego przez
Fundacje prawem autorskim, napisz do Fundacji; czasami czynimy od tego
odstepstwa. W naszej decyzji kierujemy sie dwoma celami: utrzymania
wolnego statusu wszystkich pochodnych naszego wolnego oprogramowania
oraz - generalnie - promowania wspóludzialu i wielokrotnego stosowania
oprogramowania.


    WYRÓB BEZ GWARANCJI

  11. PONIEWAZ PROGRAM JEST LICENCJONOWANY BEZPLATNIE, NIE JEST OBJETY
GWARANCJA W ZAKRESIE DOZWOLONYM PRZEZ OBOWIAZUJACE PRZEPISY. O ILE NA
PISMIE NIE STANOWI SIE INACZEJ, POSIADACZE PRAW AUTORSKICH I/LUB INNE
STRONY ZAPEWNIAJA PROGRAM W STANIE, W JAKIM JEST ("JAK WIDAC") BEZ
JAKIEJKOLWIEK GWARANCJI, ANI WYRAZNEJ, ANI DOMYSLNEJ, W TYM MIEDZY
INNYMI DOMYSLNYCH GWARANCJI CO DO PRZYDATNOSCI HANDLOWEJ I PRZYDATNOSCI
DO OKRESLONYCH ZASTOSOWAN. CALOSC RYZYKA W ZAKRESIE JAKOSCI I
SKUTECZNOSCI DZIALANIA PROGRAMU PONOSISZ SAM. W RAZIE GDYBY PROGRAM
OKAZAL SIE WADLIWY, PONOSISZ KOSZT CALEGO NIEZBEDNEGO SERWISU, NAPRAWY
I KORYGOWANIA.

  12. O ILE OBOWIAZUJACE PRAWO NIE STANOWI INACZEJ ALBO CZEGOS INNEGO
NIE UZGODNIONO W FORMIE PISEMNEJ, ZADEN POSIADACZ PRAW AUTORSKICH ANI
INNA STRONA MODYFIKUJACA I/LUB REDYSTRYBUJACA PROGRAM ZGODNIE Z
POWYZSZYMI ZEZWOLENIAMI, W ZADNYM WYPADKU NIE JEST ODPOWIEDZIALNA WOBEC
CIEBIE ZA SZKODY, W TYM SZKODY OGÓLNE, SPECJALNE, UBOCZNE LUB SKUTKOWE,
WYNIKLE Z UZYCIA BADZ NIEMOZLIWOSCI UZYCIA PROGRAMU (W TYM, MIEDZY
INNYMI, ZA UTRATE DANYCH LUB POWSTANIE DANYCH NIEDOKLADNYCH, ALBO ZA
STRATY PONIESIONE PRZEZ CIEBIE LUB STRONY TRZECIE, JAK TEZ NIEDZIALANIE
PROGRAMU Z INNYMI PROGRAMAMI), NAWET JESLI DANY POSIADACZ BADZ INNA
STRONA ZOSTALI POWIADOMIENI O MOZLIWOSCI POWSTANIA TAKICH SZKÓD.

     KONIEC ZASAD I WARUNKÓW

    Jak stosowac niniejsze Warunki do Twoich nowych programów

  Jesli opracowujesz nowy program i chcialbys, aby stal sie on
przydatny dla szerokiego ogólu, najlepsza droga do osiagniecia tego
bedzie nadanie twemu programowi charakteru wolnego oprogramowania,
które kazdy moze redystrybuowac i zmieniac na niniejszych warunkach. 

  W tym celu do programu dolacz ponizsze informacje. Bezpieczniej jest
dolaczyc je na poczatku kazdego pliku zródlowego, dzieki czemu
najskuteczniej mozna przekazac fakt nieistnienia gwarancji; kazdy plik
powinien przy tym nosic uwage "copyright" i odnosnik, gdzie mozna
znalezc pelna informacje. 

    nazwa programu i informacja, do czego on sluzy.
    Copyright (C) 19../20.. nazwisko autora

    Niniejszy program jest wolnym oprogramowaniem; mozesz go
    rozprowadzac dalej i/lub modyfikowac na warunkach Powszechnej
    Licencji Publicznej GNU, wydanej przez Fundacje Wolnego
    Oprogramowania - wedlug wersji 2-giej tej Licencji lub którejs
    z pózniejszych wersji.


    Niniejszy program rozpowszechniany jest z nadzieja, iz bedzie on
    uzyteczny - jednak BEZ JAKIEJKOLWIEK GWARANCJI, nawet domyslnej
    gwarancji PRZYDATNOSCI HANDLOWEJ albo PRZYDATNOSCI DO OKRESLONYCH
    ZASTOSOWAN. W celu uzyskania blizszych informacji - Powszechna
    Licencja Publiczna GNU.


    Z pewnoscia wraz z niniejszym programem otrzymales tez egzemplarz
    Powszechnej Licencji Publicznej GNU (GNU General Public License);
    jesli nie - napisz do Free Software Foundation, Inc., 675 Mass Ave,
    Cambridge, MA 02139, USA.


Podaj tez informacje o sposobie kontaktowania sie z toba poczta
elektroniczna lub zwykla.

Jesli dany program jest interaktywny, spraw, aby w momencie wchodzenia
w tryb interaktywny wyswietlal on komunikat jak w ponizszym
przykladzie: 

    Gnomovision wersja 69, Copyright C 19.. nazwisko autora
    Gnomovision wydawany jest ABSOLUTNIE BEZ ZADNEJ GWARANCJI -
    w celu uzyskania dalszych szczególów wpisz "show w".
    To jest wolne oprogramowanie i mile widziane jest dalsze
    rozpowszechnianie go przez ciebie na okreslonych warunkach -
    w celu uzyskania blizszych szczególów wpisz "show c".

Powyzsze hipotetyczne polecenia "show w" i "show c" winny powodowac
wyswietlenie odpowiednich czesci Powszechnej Licencji Publicznej.
Oczywiscie mozesz uzywac innych polecen niz "show w" i "show c"; moga
to byc nawet klikniecia mysza lub pozycje menu - co tylko sobie uznasz
za stosowne. 

Powinienes tez poprosic swego pracodawce (jesli pracujesz jako
programista) czy tez swoja szkole (jesli jestes uczniem), o
podpisanie, w razie potrzeby, "Rezygnacji z praw autorskich" do
programu. Ponizej podajemy przyklad (zmien nazwy/nazwiska): 

  My, firma Jojodyne Sp. z o.o. niniejszym zrzekamy sie
  i rezygnujemy z wszelkich interesów prawnych w zakresie
  praw autorskich do programu "Gnomovision" (który realizuje
  nastepujace funkcje...), napisanego przez p.Jana Kowalskiego.

  Podpis: /-/ Gniewoslaw Wielkowazny

  Gniewoslaw Wielkowazny, Prezes...itp

Powszechna Licencja Publiczna nie zezwala na wlaczanie twego programu
do programów prawnie zastrzezonych. Jesli twój program jest biblioteka
podprogramów, mozesz rozwazyc, czy nie bedzie korzystniej zezwolic na
powiazanie prawnie zastrzezonych aplikacji z biblioteka. Jesli
chcialbys wlasnie tego dokonac, zamiast niniejszej Licencji zastosuj
Powszechna Licencje Publiczna GNU dla Bibliotek.
