#undef PROJECT_NAME
#undef ENABLE_NLS
#undef HAVE_CATGETS
#undef HAVE_GETTEXT
#undef HAVE_LC_MESSAGES
#undef HAVE_STPCPY
#undef HAVE_LIBSM
#undef PACKAGE_LOCALE_DIR
#undef PACKAGE_DATA_DIR
#undef PACKAGE_PLUGINS_DIR
#undef PACKAGE_PLUGINS_CATALOG_DIR
#undef PACKAGE_PLUGINS_DESCRIPTION_DIR
#undef PACKAGE_SOURCE_DIR
#undef GW_PROG_PATH_MOUNT
#undef GW_PROG_PATH_UMOUNT
#undef GW_DEBUG_MODE
#undef GW_DEBUG_TIME
#undef GW_DEBUG_TOOLS_COMPONENT
#undef GW_DEBUG_DATA_COMPONENT
#undef GW_DEBUG_GUI_COMPONENT
#undef GW_DEBUG_GUI_CALLBACK_COMPONENT
#undef HAVE_GTK12
#undef HAVE_GTK20
